# Ubuntu Testing Automation Harness
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test command line argument functions."""

import os.path
import shutil
import tempfile
import unittest

from argparse import ArgumentTypeError
from jsonschema import ValidationError
from mock import patch, Mock

from utah.config import config
from utah.client.exceptions import (
    YAMLEmptyFile,
    YAMLParsingError,
)
from utah.run import (
    _dump_preseed,
    _write,
    is_utah_done,
    master_runlist_argument,
)


class TestMasterRunlistArgument(unittest.TestCase):

    """Test master_runlist_argument function."""

    def setUp(self):
        """Peform temporary setup required by tests."""
        self.logpath = config.logpath
        config.logpath = tempfile.mkdtemp(prefix='utah_run')

    def tearDown(self):
        """Undo temporary stuff from setUp."""
        shutil.rmtree(config.logpath)
        config.logpath = self.logpath

    @patch('utah.run.parse_yaml_file')
    @patch('utah.run.url_argument')
    def test_empty_file(self, url_argument, parse_yaml_file):
        """ArgumentTypeError is raised on empty file."""
        parse_yaml_file.side_effect = YAMLEmptyFile()
        pattern = r'Master runlist seems to be empty:'
        with self.assertRaisesRegexp(ArgumentTypeError, pattern):
            master_runlist_argument('url')

    @patch('utah.run.parse_yaml_file')
    @patch('utah.run.url_argument')
    def test_parsing_error(self, url_argument, parse_yaml_file):
        """ArgumentTypeError is raised on parsing error."""
        parse_yaml_file.side_effect = YAMLParsingError()
        pattern = r'Master runlist failed to parse as a yaml file:'
        with self.assertRaisesRegexp(ArgumentTypeError, pattern):
            master_runlist_argument('url')

    @patch('utah.run.jsonschema.validate')
    @patch('utah.run.parse_yaml_file')
    @patch('utah.run.url_argument')
    def test_validation_error(self, url_argument, parse_yaml_file, validate):
        """ArgumentTypeError is raised on validation error."""
        validate.side_effect = ValidationError('error description')
        pattern = 'Master runlist failed to validate:'
        with self.assertRaisesRegexp(ArgumentTypeError, pattern):
            master_runlist_argument('url')

    @patch('utah.run.jsonschema.validate')
    @patch('utah.run.parse_yaml_file')
    @patch('utah.run.url_argument')
    def test_filename_returned(self, url_argument, parse_yaml_file, validate):
        """Filename is returned on success."""
        expected_filename = 'filename'
        url_argument.side_effect = [expected_filename]
        filename = master_runlist_argument('url')
        self.assertEqual(filename, expected_filename)


class TestRun(unittest.TestCase):

    """Unit tests on utility functions in run.py."""

    def setUp(self):
        """Set common filename."""
        self.preseed_file = '/tmp/name-preseed.cfg'

    def tearDown(self):
        """Remove any leftover files."""
        if os.path.isfile(self.preseed_file):
            os.unlink(self.preseed_file)

    def test__write(self):
        """Ensure proper behavior of the _write function."""
        logs = []
        args = Mock()
        args.configure_mock(
            report_type='yaml', runlist='tmp.txt', dumplogs=True)
        machine = Mock()
        machine.name = 'name'
        report_path = '/path/to/report'
        _write(machine, args, logs, report_path)
        self.assertTrue(machine.downloadfiles.called)
        call_args, _call_kwargs = machine.downloadfiles.call_args
        self.assertListEqual([call_args[1]], logs)
        self.assertEqual(call_args[0], report_path)

    def test_dont_dump_preseed(self):
        """Ensure _copy_preseed respects the argument."""
        logs = []
        args = Mock()
        args.outputpreseed = False
        machine = Mock()
        machine.configure_mock(name='name')
        _dump_preseed(machine, args, logs)
        self.assertEqual(0, len(logs))
        self.assertFalse(machine.preseed.dump.called)

    def test_dump_preseed(self):
        """Ensure _copy_preseed is called when needed."""
        logs = []
        args = Mock()
        args.outputpreseed = True
        preseed = Mock()
        preseed.configure_mock(dump=open('/dev/null').read)
        machine = Mock()
        machine.configure_mock(name='name', preseed=preseed)
        config = Mock()
        config.logpath = os.path.dirname(self.preseed_file)
        with patch('utah.run.config', config) as _mock_config:
            _dump_preseed(machine, args, logs)
            self.assertEqual(1, len(logs))
            self.assertTrue(os.path.isfile(self.preseed_file))

    def test_is_utah_done(self):
        """Ensure the utah_is_done logic works correctly with exceptions."""
        machine = Mock()
        machine.run.side_effect = Exception('blah')
        with self.assertRaises(Exception) as cm:
            is_utah_done(machine, 1)

        self.assertEquals(cm.exception.message, 'blah')
        self.assertTrue(cm.exception.retry)
